#!/bin/bash
#
# Bootstrap script called by servermill during cloud server setup. Not
# intended to be run by end users.
#
# To match servermill's behavior, this script expects the environment
# variables USERNAME, DATACENTER, and API_KEY, which are passed to the
# corresponding arguments of driveclient --configure. AUTH_TOKEN may
# substitute for API_KEY if desired.
#
# For testing purposes, you may also supply any of the following. Note
# these are not Officially Supported and should not be relied on in
# production:
#
# CBU_AGENTREPO=file://path/to/repo
# FLAVOR=driveclient-flavor-argument
# CBU_EXTRA_ARGS="--arg1 --arg2"

# FIXME: for debugging, remove in release
set -e
set -u
# Since these are either-or, provide an empty default so set -u doesn't
# trip on whichever one you're not using.
API_KEY=${API_KEY:-}
AUTH_TOKEN=${AUTH_TOKEN:-}
CBAU_EXTRA_ARGS=${CBAU_EXTRA_ARGS:-}

export DEBIAN_FRONTEND=noninteractive  # shut up, apt
# https://manpages.debian.org/bullseye/debconf-doc/debconf.7.en.html#Frontends

pkgcmd () {
	for cmd in apt-get apt yum; do
		command -v $cmd 2>/dev/null && return
	done
}

installdeps () {
	local cmd
	cmd=$(pkgcmd)
	if [ -z "${cmd}" ]; then
		echo "failed to install missing sysdeps (can't find package manager)"
		return 1
	fi
	echo "installing missing sysdeps: $*"
	if [[ $cmd  == */apt* ]]; then
		$cmd update
	fi
	$cmd install -y "$@"
}

precheck () {
	# pre-setup sanity checks
	local req_cmds="curl python3"
	local req_vars="USERNAME DATACENTER"
	local missing_cmds=""
	local missing_vars=""
	local status=0

	for cmd in ${req_cmds}; do
		# Note trailing space here and below
		type "$cmd" > /dev/null || missing_cmds="${missing_cmds}${cmd} "
	done
	for var in ${req_vars}; do
		[ -z "${!var}" ] && missing_vars="${missing_vars}${var} "
	done
	if [ -z "${API_KEY}${AUTH_TOKEN}" ]; then
		missing_vars="${missing_vars}(API_KEY|AUTH_TOKEN) "
	fi

	if [ -n "${missing_cmds}" ]; then
		echo "missing required commands: ${missing_cmds}"
		# shellcheck disable=SC2086 # intentional, but should it be?
		installdeps ${missing_cmds}
		status=$?
	fi
	if [ -n "${missing_vars}" ]; then
		echo "missing required env vars: ${missing_vars}"
		status=1
	fi

	if [ "$EUID" -ne 0 ]; then
		local lines=("WARNING: we are not root. This is allowed for"
		             "testing purposes, but you should expect driveclient"
		             "--configure to fail")
		echo "${lines[*]}"
	fi

	return $status
}


bootstrap_updater () {
	# Allow overriding the repo path for local testing purposes
	local repo="${CBU_AGENTREPO:-https://agentrepo.${DATACENTER}.cbu.rackspace.net/repo}"
	local script=cbu-updater-standalone.py
	local url=${repo}/${script}
	local path=${TMP}/${script}

	# Simplest way of getting a useful indicator of a preinstalled version
	type cbu-updater >/dev/null 2>&1 && cbu-updater install cbu-updater && return 0
	echo "downloading standalone installer from ${url} to ${path}"
	# FIXME: the redirect here stops curl from inserting a bare carriage
	# return in the log. I am not sure if this hides legitimate errors
	# -- those *should* be on stderr, but verify.
	#
	# It might be simpler to just pipe curl's output to python, though
	# I'm not sure how to handle errors properly that way.
	curl "${url}" \
		--output "${path}" \
		--silent \
		--show-error \
		--fail \
		> /dev/null || return
	# shellcheck disable=SC2086 # intentional, but should it be?
	python3 "${path}" install cbu-updater ${CBAU_EXTRA_ARGS} || return
}

kicksvc () {
	local svc="$1"
	systemctl restart "${svc}" ||
		service "${svc}" restart ||
		/etc/init.d/"${svc}" restart
}

register () {
	local autharg
	local authval
	local conf=/etc/driveclient/bootstrap.json

	if [ -f ${conf} ]; then
		echo "skipping agent configuration (agent already configured)"
		return 0
	fi

	if [ -n "${AUTH_TOKEN}" ]; then
		autharg=authtoken
		authval=${AUTH_TOKEN}
	elif [ -n "${API_KEY}" ]; then
		autharg=apikey
		authval=${API_KEY}
	else
		# shouldn't happen, precheck should catch this
		echo "must supply at least one of AUTH_TOKEN or API_KEY"
		return 1
	fi

	echo "configuring driveclient for ${USERNAME} in ${DATACENTER}"
	driveclient \
		--configure \
		--noninteractive \
		--flavor "${FLAVOR:-raxcloudserver}" \
		--datacenter "${DATACENTER}" \
		--user "${USERNAME}" \
		--${autharg} "${authval}" \
		|| return
		# --apihost api.${DATACENTER}.cbu.rackspace.net
		# may not need to specify apihost; it should get the right one
		# from the service catalog, I think?
	kicksvc driveclient > /dev/null 2>&1
}

fail () {
	# Shorthand to echo a message and then exit with an error
	local msg=${1}
	echo "${msg}"
	exit 1;
}

main () {
	echo "bootstrapping cloud backup agent and updater"
	precheck                 || fail "precheck failure, aborting"
	bootstrap_updater        || fail "couldn't bootstrap updater, aborting"
	cbu-updater install \
		driveclient          || fail "failed to install driveclient, aborting"
	if register; then
		:  # noop
	elif [[ $? -eq 139 ]]; then
		echo "CBU-1003 segfault bug detected; attempting nscd workaround"
		installdeps nscd || fail "problem installing nscd (returned $?)"
		echo "retrying driveclient configuration"
		register || fail "agent configuration still failed; aborting"
	else
		fail "failed to install driveclient, aborting"
	fi
}

# Haven't found a way around the global here...
TMP="$(mktemp -d)" || fail "Failed to create temporary directory"
trap 'rm -rvf -- "$TMP"' EXIT
main >&2
